#!/bin/bash
#  $HeadURL: https://svn.pasta.freemyip.com/main/paa/tags/4/tests/bin/PAA121_PAA-2.sh $ $LastChangedRevision: 10280 $
PROGNAME=`basename $0`
SANDPIT=`pwd`
. $(ade-config ade_share_prefix)/include/adetestsupport.sh

##############################################################################
#                           
#  Purpose of test: to verify basic paa functions
#
#  Bug id:          <none>
#
##############################################################################

#  Ensure dates are consistently formatted
export LC_ALL=C

#  Start of support functions
filter_variant_text()
{
    #  Note how the variable *width* of a hostname - and therefore of the padding
    #  that follows it in column-aligned output - means that we can't change 
    #  just <hostname> to VARIANT-HOSTNAME but <hostname> *with* padding to
    #  VARIANT-HOSTNAME with fixed width padding. Another padding issues comes
    #  from that summer and winter timezones have different widths and therefore
    #  different paddings. Although the timezone is devarianted the spacing is
    #  not. This only seems to be a problem on the end of the line because dates
    #  are always in the last column.
    eval "$@" | sed -e 's/[A-Z][a-z][a-z] [A-Z][a-z][a-z] [1-3 ][0-9] [0-2][0-9]:[0-5][0-9]:[0-5][0-9] [A-Z][A-Z]* [1-2][0-9][0-9][0-9] *$/Fri Dec 31 23:59:59 GMT 1999/g' \
                    -e 's/^\(pub\)\( *\)\([^ ]*\)\( *\)\([^ ]*\)$/\1\2VARIANT-KEY-ID\4VARIANT-DATE/' \
                    -e "s@$SANDPIT@VARIANT-SANDPIT@g" \
                    -e "s/$UNAMEN  */VARIANT-HOSTNAME /g" \
                    -e "s/$UNAMEN\([^ \t]\)/VARIANT-HOSTNAME\1/g" \
                    -e "s/20[1-9][0-9][0-1][0-9][0-3][0-9][0-2][0-9][0-5][0-9][0-5][0-9]/VARIANT-TIMESTAMP/g" \
		    -e "s/  *\$//"

    #  return the passed command's return code, not sed's.
    return ${PIPESTATUS[0]}
}

launch_shell()
{
    PS1="$PROGNAME> " bash --norc || true
}

echo "$PROGNAME: configuring environment ..."
#  Ensure DB is created in sandpit
export PAA_RCDIR=$SANDPIT/.paa
#  Ensure effects on OS (modiying the *OS*'s idea of repos, restarting apache, etc) are not real
export PAA_ROOTDIR=$SANDPIT
#  Force use of non-interactive editor
export EDITOR=$SANDPIT/bin/caton
#  Use a test-specific gpg configuration
export GNUPGHOME=$SANDPIT/.gpupg
#  Don't use any running gpg-agent
unset GPG_AGENT_INFO
#  Don't put the apache config files in $MODROOT/var 'cos they'll contaminate it.
export PAA_STATE_PREFIX=$SANDPIT/var
#  Misc
UNAMEN=$(uname -n)
#  Beware that the database we'll copy into place shortly assumes that the hostname 
#  of *THIS* machine is 'gnocchi'. Therefore we need to tell paa that.
export PAA_HOSTNAME=gnocchi

echo "$PROGNAME: installing pre-prepared paa.sqlite file ..."
mkdir -p $PAA_RCDIR
cp $ADETEST_MODROOT/tests/in/PAA121_PAA-2.sqlite $PAA_RCDIR/paa.sqlite
mkdir -p $PAA_ROOTDIR/etc/apt/sources.list.d
ACCESSES=$(paa listaccesses | sed -e '1,2d' -e '$d' | sed 's/ .*//')
for ACCESS in $ACCESSES; do
    echo "dummy text" > $PAA_ROOTDIR/etc/apt/sources.list.d/$ACCESS.list
done

echo "$PROGNAME: removing one access ..."
paa unaccess debian-security
